/*
   ; Fonctionnement par Interruption(tests)
   ; ======================================
   ;------------------------------------------------------------------------
   ; Author:    Olivier Carmona.        1999-2001 (C) K-Team S.A.
   ;
   ; Project:   Kameleon
   ; Goal: Interruption generation using the CTM (MCSM2 aor FCSM12 submodule)
   ; 
   ;------------------------------------------------------------------------
*/


#include <sys/kos.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>


/* Variable that must be the first one */
static unsigned long vIDProcess[1];

/*
   ;  FCSM Module Routines
   ;------------------------------------------------------------------------
*/

void enable_irq_fcsm(void);
void disable_irq_fcsm(void);
void loop_fcsm(void);

static unsigned int typical_time_fcsm = 0;
static unsigned char cpt_fcsm1 = 0, cpt_fcsm2 = 0;

asm("
loop_fcsm: 
        movem.l a0-a6/d0-d7,-(sp)
	jsr loop_fcsmC(pc)
        movem.l (sp)+,a0-a6/d0-d7
	rte
");

void loop_fcsmC(void)
{
 unsigned int last_ticcount;
 static unsigned int ticcount;

 disable_irq_fcsm();
 if (cpt_fcsm1++ == 0) 
 {
  last_ticcount = ticcount;
  ticcount = tim_get_ticcount();
  typical_time_fcsm = ticcount - last_ticcount;
  /* Led 2 and 3 alternatively blinks */
  var_change_led(2+(cpt_fcsm2++)%2);
 }
}

/*
   ;  MCSM Module Routines
   ;------------------------------------------------------------------------
*/
void enable_irq_mcsm(void);
void disable_irq_mcsm(void);
void loop_mcsm(void);

static unsigned int typical_time_mcsm = 0;
static unsigned char cpt_mcsm1 = 0, cpt_mcsm2 = 0;

asm("
loop_mcsm: 
        movem.l a0-a6/d0-d7,-(sp)
	jsr loop_mcsmC(pc)
        movem.l (sp)+,a0-a6/d0-d7
	rte
");

void loop_mcsmC(void)
{
 unsigned int last_ticcount;
 static unsigned int ticcount;

 disable_irq_mcsm();
 if (cpt_mcsm1++ == 0) 
 {
  last_ticcount = ticcount;
  ticcount = tim_get_ticcount();
  typical_time_mcsm = ticcount - last_ticcount;
  /* Led 0 and 1 alternatively blinks */
  var_change_led((cpt_mcsm2++)%2);
 }
}

/* Process 0
 * ^^^^^^^^^
 */

static void
process_0 ()
{
 /* Interrupt Number = Default Base for CTM (0xC0) + Module Number for MCSM (0x2)*/
 var_set_exception(loop_mcsm, 0xC2);

 /* Interrupt Number = Default Base for CTM (0xC0) + Module Number for FCSM (0xC)*/
 var_set_exception(loop_fcsm, 0xCC);
 
 /* Activate FCSM interrupt */
 enable_irq_fcsm();

 /* Activate MCSM interrupt */
 enable_irq_mcsm();
 
 for (;;)
  {
   sleep(100);
   printf("Time FCSM %f ms / %d iter.; MCSM %f ms / %d iter.",
      (float)typical_time_fcsm/256,cpt_fcsm1,
      (float)typical_time_mcsm/256,cpt_mcsm1);
  }
}


/* MAIN
 * ====
 *
 * - Initialize the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

main (void)
{
 int32 status;
 static char prName_0[] = "Interrupt test, OC-97          Rev. 1.00\r\n";

 status = install_task (prName_0, 1000, process_0);
 if (status == -1)
   exit (0);
 vIDProcess[0] = (unsigned long) status;
}
